------------------------------------------------------------------------------
-- user_logic.vhd - entity/architecture pair
------------------------------------------------------------------------------
--
-- ***************************************************************************
-- ** Copyright (c) 1995-2009 Xilinx, Inc.  All rights reserved.            **
-- **                                                                       **
-- ** Xilinx, Inc.                                                          **
-- ** XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"         **
-- ** AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND       **
-- ** SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,        **
-- ** OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,        **
-- ** APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION           **
-- ** THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,     **
-- ** AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE      **
-- ** FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY              **
-- ** WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE               **
-- ** IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR        **
-- ** REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF       **
-- ** INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS       **
-- ** FOR A PARTICULAR PURPOSE.                                             **
-- **                                                                       **
-- ***************************************************************************
--
------------------------------------------------------------------------------
-- Filename:          user_logic.vhd
-- Version:           1.00.a
-- Description:       User logic.
-- Date:              Thu Aug 20 17:11:58 2009 (by Create and Import Peripheral Wizard)
-- VHDL Standard:     VHDL'93
------------------------------------------------------------------------------
-- Naming Conventions:
--   active low signals:                    "*_n"
--   clock signals:                         "clk", "clk_div#", "clk_#x"
--   reset signals:                         "rst", "rst_n"
--   generics:                              "C_*"
--   user defined types:                    "*_TYPE"
--   state machine next state:              "*_ns"
--   state machine current state:           "*_cs"
--   combinatorial signals:                 "*_com"
--   pipelined or register delay signals:   "*_d#"
--   counter signals:                       "*cnt*"
--   clock enable signals:                  "*_ce"
--   internal version of output port:       "*_i"
--   device pins:                           "*_pin"
--   ports:                                 "- Names begin with Uppercase"
--   processes:                             "*_PROCESS"
--   component instantiations:              "<ENTITY_>I_<#|FUNC>"
------------------------------------------------------------------------------

-- DO NOT EDIT BELOW THIS LINE --------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library proc_common_v3_00_a;
use proc_common_v3_00_a.proc_common_pkg.all;

-- DO NOT EDIT ABOVE THIS LINE --------------------

--USER libraries added here

------------------------------------------------------------------------------
-- Entity section
------------------------------------------------------------------------------
-- Definition of Generics:
--   C_SLV_DWIDTH                 -- Slave interface data bus width
--   C_NUM_REG                    -- Number of software accessible registers
--
-- Definition of Ports:
--   Bus2IP_Clk                   -- Bus to IP clock
--   Bus2IP_Reset                 -- Bus to IP reset
--   Bus2IP_Data                  -- Bus to IP data bus
--   Bus2IP_BE                    -- Bus to IP byte enables
--   Bus2IP_RdCE                  -- Bus to IP read chip enable
--   Bus2IP_WrCE                  -- Bus to IP write chip enable
--   IP2Bus_Data                  -- IP to Bus data bus
--   IP2Bus_RdAck                 -- IP to Bus read transfer acknowledgement
--   IP2Bus_WrAck                 -- IP to Bus write transfer acknowledgement
--   IP2Bus_Error                 -- IP to Bus error response
------------------------------------------------------------------------------

entity user_logic is
  generic
  (
    -- ADD USER GENERICS BELOW THIS LINE ---------------
    --USER generics added here
    -- ADD USER GENERICS ABOVE THIS LINE ---------------

    -- DO NOT EDIT BELOW THIS LINE ---------------------
    -- Bus protocol parameters, do not add to or delete
    C_SLV_DWIDTH                   : integer              := 32;
    C_NUM_REG                      : integer              := 16
    -- DO NOT EDIT ABOVE THIS LINE ---------------------
  );
  port
  (
   -- ADD USER PORTS BELOW THIS LINE ------------------
    BITCLK	:	in STD_LOGIC;
	 SDATA_IN	:	in STD_LOGIC; 	-- AC Link input frame signal
	 SDATA_OUT:	out STD_LOGIC;	-- AC Link output frame signal
	 SYNC		:	out STD_LOGIC;	-- the SYNC line determines the start of a nwe frame
												-- it should have a frequency of 48KHz, regardless of the sample rate
    RESET_n	:  out STD_LOGIC;	-- AC97 cold reset signal
    -- ADD USER PORTS ABOVE THIS LINE ------------------

    -- DO NOT EDIT BELOW THIS LINE ---------------------
    -- Bus protocol ports, do not add to or delete
    Bus2IP_Clk                     : in  std_logic;
    Bus2IP_Reset                   : in  std_logic;
    Bus2IP_Data                    : in  std_logic_vector(0 to C_SLV_DWIDTH-1);
    Bus2IP_BE                      : in  std_logic_vector(0 to C_SLV_DWIDTH/8-1);
    Bus2IP_RdCE                    : in  std_logic_vector(0 to C_NUM_REG-1);
    Bus2IP_WrCE                    : in  std_logic_vector(0 to C_NUM_REG-1);
    IP2Bus_Data                    : out std_logic_vector(0 to C_SLV_DWIDTH-1);
    IP2Bus_RdAck                   : out std_logic;
    IP2Bus_WrAck                   : out std_logic;
    IP2Bus_Error                   : out std_logic
    -- DO NOT EDIT ABOVE THIS LINE ---------------------
  );

  attribute SIGIS : string;
  attribute SIGIS of Bus2IP_Clk    : signal is "CLK";
  attribute SIGIS of Bus2IP_Reset  : signal is "RST";

end entity user_logic;

------------------------------------------------------------------------------
-- Architecture section
------------------------------------------------------------------------------

architecture IMP of user_logic is

  --USER signal declarations added here, as needed for user logic

  ------------------------------------------
  -- Signals for user logic slave model s/w accessible register example
  ------------------------------------------
  signal ADDR                             : std_logic_vector(0 to C_SLV_DWIDTH-1);
  signal CONFIG_DATA                      : std_logic_vector(0 to C_SLV_DWIDTH-1);
  signal TAG_AND_ID                       : std_logic_vector(0 to C_SLV_DWIDTH-1);
  signal PCM_OUT_L                        : std_logic_vector(0 to C_SLV_DWIDTH-1);
  signal PCM_OUT_R                        : std_logic_vector(0 to C_SLV_DWIDTH-1);
  signal PCM_IN_L                         : std_logic_vector(0 to C_SLV_DWIDTH-1);
  signal PCM_IN_R                         : std_logic_vector(0 to C_SLV_DWIDTH-1);
  signal CONTROL                          : std_logic_vector(0 to C_SLV_DWIDTH-1);
  signal STATUS                           : std_logic_vector(0 to C_SLV_DWIDTH-1);
  signal slv_reg9                         : std_logic_vector(0 to C_SLV_DWIDTH-1);
  signal slv_reg10                      : std_logic_vector(0 to C_SLV_DWIDTH-1);
  signal slv_reg11                      : std_logic_vector(0 to C_SLV_DWIDTH-1);
  signal slv_reg12                      : std_logic_vector(0 to C_SLV_DWIDTH-1);
  signal slv_reg13                      : std_logic_vector(0 to C_SLV_DWIDTH-1);
  signal slv_reg14                      : std_logic_vector(0 to C_SLV_DWIDTH-1);
  signal slv_reg15                      : std_logic_vector(0 to C_SLV_DWIDTH-1);
  signal slv_reg_write_sel              : std_logic_vector(0 to 15);
  signal slv_reg_read_sel               : std_logic_vector(0 to 15);
  signal slv_ip2bus_data                : std_logic_vector(0 to C_SLV_DWIDTH-1);
  signal slv_read_ack                   : std_logic;
  signal slv_write_ack                  : std_logic;

  signal SLOT_VALID: STD_LOGIC_VECTOR (0 to 4); --inicates which slot read contains valid data
  signal SLOT_REQ	: 	STD_LOGIC_VECTOR (0 to 1); --read tag/slot request bits
  signal ADDR_OUT	:	STD_LOGIC_VECTOR (0 to 6); --Address from which is read
  signal DATA_OUT	:	STD_LOGIC_VECTOR (0 to 15);--Data read from the register
  signal PCM_IN_L_REC	:	STD_LOGIC_VECTOR (0 to 19);--left channel PCM input data;
  signal PCM_IN_R_REC	:	STD_LOGIC_VECTOR (0 to 19);--right channel PCM output data;
  signal DONE:   STD_LOGIC;
  
  signal LOAD_CMB: STD_LOGIC;
  type STATES_LOAD is (Idle_Load, Wait_on_Done, Clear_Load_Status);
  signal STC_LOAD, STN_LOAD: STATES_LOAD :=Idle_Load;
   
component AC97_Core 
    Port ( 
			-- Input Ports
			TAG_IN	: 	in STD_LOGIC_VECTOR (4 downto 0);  --write the tag information for the AC97 frame
			ID			:	in STD_LOGIC_VECTOR (1 downto 0);  --AC97 codec ID
			RNW		:	in STD_LOGIC;							  --read from or write to a specific AC97 codec internal register
			ADDR		:	in STD_LOGIC_VECTOR (6 downto 0);  --AC97 codec internal register address
			DATA_WR	:	in STD_LOGIC_VECTOR (15 downto 0); --Data to write into the register
																		  --If a read operation is performed, the data sent should be '0'-s
			PCM_OUT_L:	in STD_LOGIC_VECTOR (19 downto 0); --Left channel PCM output data
			PCM_OUT_R:	in STD_LOGIC_VECTOR (19 downto 0); --Right channel PCM output data
			-- output ports
			SLOT_VALID: out STD_LOGIC_VECTOR (4 downto 0); --inicates which slot read contains valid data
         SLOT_REQ	: 	out STD_LOGIC_VECTOR (1 downto 0); --read tag/slot request bits
			ADDR_OUT	:	out STD_LOGIC_VECTOR (6 downto 0); --Address from which is read
			DATA_OUT	:	out STD_LOGIC_VECTOR (15 downto 0);--Data read from the register
			PCM_IN_L	:	out STD_LOGIC_VECTOR (19 downto 0);--left channel PCM input data;
			PCM_IN_R	:	out STD_LOGIC_VECTOR (19 downto 0);--right channel PCM output data;
			-- AC 97 link ports
			-- The bitclk signal is input for an AC97 primary codec (in the case when the XUPV2P board is used)
			-- and an output signal to the AC97 secondary codec (in the case when the XUPV2P board is used)
			-- SDATA_OUT is changed on the positive edge of BITCLK and SDATA_IN is read on the negative edge of BITCLK
			BITCLK	:	in STD_LOGIC;
			SDATA_IN	:	in STD_LOGIC; 	-- AC Link input frame signal
			SDATA_OUT:	out STD_LOGIC;	-- AC Link output frame signal
			SYNC		:	out STD_LOGIC;	-- the SYNC line determines the start of a nwe frame
												-- it should have a frequency of 48KHz, regardless of the sample rate
			RESET_n	:  out STD_LOGIC;	-- AC97 cold reset signal
			-- System and control ports
			SYSCLK 	: 	in  STD_LOGIC;	-- System Clock
			RESET		:	in STD_LOGIC;	-- Reset input
			--BITCLK_IN:	in STD_LOGIC; --should have a frequency of 12.288MHz
			LOAD		:	in STD_LOGIC;	--Load a new frame
			DONE		:  out STD_LOGIC); -- The loaded frame was transmitted
end component;


begin

AC97_CORE_I: AC97_Core
    port map
    (
			-- Input Ports
			TAG_IN	   => TAG_AND_ID  (C_SLV_DWIDTH - 7 to C_SLV_DWIDTH - 3),  --write the tag information for the AC97 frame
			ID			   => TAG_AND_ID  (C_SLV_DWIDTH - 2 to C_SLV_DWIDTH - 1), --AC97 codec ID
			RNW		   => CONTROL (C_SLV_DWIDTH - 2), --read from or write to a specific AC97 codec internal register
			ADDR		   => ADDR (C_SLV_DWIDTH - 7 to C_SLV_DWIDTH - 1),  --AC97 codec internal register address
			DATA_WR	   => CONFIG_DATA (C_SLV_DWIDTH - 16 to C_SLV_DWIDTH -1), --Data to write into the register
																		  --If a read operation is performed, the data sent should be '0'-s
			PCM_OUT_L   => PCM_OUT_L (C_SLV_DWIDTH - 20 to C_SLV_DWIDTH - 1), --Left channel PCM output data
			PCM_OUT_R   => PCM_OUT_R (C_SLV_DWIDTH - 20 to C_SLV_DWIDTH - 1), --Right channel PCM output data
			-- output ports
			SLOT_VALID  => SLOT_VALID, --inicates which slot read contains valid data
         SLOT_REQ	   => SLOT_REQ,  --read tag/slot request bits
			ADDR_OUT    => ADDR_OUT, --Address from which is read
			DATA_OUT    => DATA_OUT, --Data read from the register
			PCM_IN_L    => PCM_IN_L_REC, --left channel PCM input data;
			PCM_IN_R	   => PCM_IN_R_REC,--right channel PCM output data;
			-- AC 97 link ports
			-- The bitclk signal is input for an AC97 primary codec (in the case when the XUPV2P board is used)
			-- and an output signal to the AC97 secondary codec (in the case when the XUPV2P board is used)
			-- SDATA_OUT is changed on the positive edge of BITCLK and SDATA_IN is read on the negative edge of BITCLK
			BITCLK	   =>    BITCLK,
			SDATA_IN	   =>    SDATA_IN, 	-- AC Link input frame signal
			SDATA_OUT   =>    SDATA_OUT,	-- AC Link output frame signal
			SYNC		   =>    SYNC,	-- the SYNC line determines the start of a nwe frame
												-- it should have a frequency of 48KHz, regardless of the sample rate
			RESET_n	   =>    RESET_n, -- AC97 cold reset signal
			-- System and control ports
			SYSCLK 	   =>    Bus2IP_Clk,	-- System Clock
			RESET		   =>    Bus2IP_Reset,	-- Reset input
			--BITCLK_IN:	in STD_LOGIC; --should have a frequency of 12.288MHz
			LOAD		  =>  LOAD_CMB,	--Load a new frame
			DONE       => DONE); -- The loaded frame was transmitted and a new frame was received



  --USER logic implementation added here

LOAD_CMB <= CONTROL (C_SLV_DWIDTH - 1) AND DONE;

--register states process
reg_states: process (Bus2IP_Clk, Bus2IP_Reset, STN_LOAD) is
begin
   if Bus2IP_Clk'EVENT and Bus2IP_Clk = '1' then
      if Bus2IP_Reset = '1' then
         STC_LOAD<=Idle_Load;
      else
         STC_LOAD<=STN_LOAD;      
      end if;
   end if;
end process reg_states;

--Transitions process
tranz: process(STC_LOAD, DONE, LOAD_CMB) is
begin 
   STN_LOAD <= STC_LOAD;
   case STC_LOAD is
   when  Idle_Load            => if LOAD_CMB = '1' then STN_LOAD <= Wait_on_Done;
                                 end if;
   when  Wait_on_Done         => if DONE = '1' then STN_LOAD <= Clear_Load_Status;
                                 end if;
   when  Clear_Load_Status    => STN_LOAD <= Idle_Load;
   when others => STN_LOAD <= Idle_Load;
   end case;
end process tranz;


  ------------------------------------------
  -- Example code to read/write user logic slave model s/w accessible registers
  -- 
  -- Note:
  -- The example code presented here is to show you one way of reading/writing
  -- software accessible registers implemented in the user logic slave model.
  -- Each bit of the Bus2IP_WrCE/Bus2IP_RdCE signals is configured to correspond
  -- to one software accessible register by the top level template. For example,
  -- if you have four 32 bit software accessible registers in the user logic,
  -- you are basically operating on the following memory mapped registers:
  -- 
  --    Bus2IP_WrCE/Bus2IP_RdCE   Memory Mapped Register
  --                     "1000"   C_BASEADDR + 0x0
  --                     "0100"   C_BASEADDR + 0x4
  --                     "0010"   C_BASEADDR + 0x8
  --                     "0001"   C_BASEADDR + 0xC
  -- 
  ------------------------------------------
  slv_reg_write_sel <= Bus2IP_WrCE(0 to 15);
  slv_reg_read_sel  <= Bus2IP_RdCE(0 to 15);
  slv_write_ack     <= Bus2IP_WrCE(0) or Bus2IP_WrCE(1) or Bus2IP_WrCE(2) or Bus2IP_WrCE(3) or Bus2IP_WrCE(4) or Bus2IP_WrCE(5) or Bus2IP_WrCE(6) or Bus2IP_WrCE(7) or Bus2IP_WrCE(8) or Bus2IP_WrCE(9) or Bus2IP_WrCE(10) or Bus2IP_WrCE(11) or Bus2IP_WrCE(12) or Bus2IP_WrCE(13) or Bus2IP_WrCE(14) or Bus2IP_WrCE(15);
  slv_read_ack      <= Bus2IP_RdCE(0) or Bus2IP_RdCE(1) or Bus2IP_RdCE(2) or Bus2IP_RdCE(3) or Bus2IP_RdCE(4) or Bus2IP_RdCE(5) or Bus2IP_RdCE(6) or Bus2IP_RdCE(7) or Bus2IP_RdCE(8) or Bus2IP_RdCE(9) or Bus2IP_RdCE(10) or Bus2IP_RdCE(11) or Bus2IP_RdCE(12) or Bus2IP_RdCE(13) or Bus2IP_RdCE(14) or Bus2IP_RdCE(15);

  -- implement slave model software accessible register(s)
  SLAVE_REG_WRITE_PROC : process( Bus2IP_Clk ) is
  begin

    if Bus2IP_Clk'event and Bus2IP_Clk = '1' then
      if Bus2IP_Reset = '1' then
		  ADDR         <= (others => '0');
        CONFIG_DATA      <= (others => '0');
        TAG_AND_ID   <= (others => '0');
        PCM_OUT_L    <= (others => '0');
        PCM_OUT_R    <= (others => '0');
        PCM_IN_L     <= (others => '0');
        PCM_IN_R     <= (others => '0');
        CONTROL      <= (others => '0');
        STATUS       <= (others => '0');
        slv_reg9     <= (others => '0');
        slv_reg10 <= (others => '0');
        slv_reg11 <= (others => '0');
        slv_reg12 <= (others => '0');
        slv_reg13 <= (others => '0');
        slv_reg14 <= (others => '0');
        slv_reg15 <= (others => '0');
      else
        case slv_reg_write_sel is
          when "1000000000000000" =>
          for byte_index in 0 to (C_SLV_DWIDTH/8)-1 loop
              if ( Bus2IP_BE(byte_index) = '1' ) then
                ADDR(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
              end if;
            end loop;
          when "0100000000000000" =>
          for byte_index in 0 to (C_SLV_DWIDTH/8)-1 loop
              if ( Bus2IP_BE(byte_index) = '1' ) then
                CONFIG_DATA(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
              end if;
            end loop;
          when "0010000000000000" =>
             for byte_index in 0 to (C_SLV_DWIDTH/8)-1 loop
              if ( Bus2IP_BE(byte_index) = '1' ) then
                TAG_AND_ID(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
              end if;
            end loop;
          when "0001000000000000" =>
            for byte_index in 0 to (C_SLV_DWIDTH/8)-1 loop
              if ( Bus2IP_BE(byte_index) = '1' ) then
                PCM_OUT_L(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
              end if;
            end loop;
          when "0000100000000000" =>
            for byte_index in 0 to (C_SLV_DWIDTH/8)-1 loop
              if ( Bus2IP_BE(byte_index) = '1' ) then
                PCM_OUT_R(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
              end if;
            end loop;
          when "0000010000000000" =>
            for byte_index in 0 to (C_SLV_DWIDTH/8)-1 loop
              if ( Bus2IP_BE(byte_index) = '1' ) then
                PCM_IN_L(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
              end if;
            end loop;
          when "0000001000000000" =>
             for byte_index in 0 to (C_SLV_DWIDTH/8)-1 loop
              if ( Bus2IP_BE(byte_index) = '1' ) then
                PCM_IN_R(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
              end if;
            end loop;
          when "0000000100000000" =>
             for byte_index in 0 to (C_SLV_DWIDTH/8)-1 loop
              if ( Bus2IP_BE(byte_index) = '1' ) then
                CONTROL(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
              end if;
            end loop;
          when "0000000010000000" =>
            for byte_index in 0 to (C_SLV_DWIDTH/8)-1 loop
              if ( Bus2IP_BE(byte_index) = '1' ) then
                STATUS(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
              end if;
            end loop;
          when "0000000001000000" =>
             for byte_index in 0 to (C_SLV_DWIDTH/8)-1 loop
              if ( Bus2IP_BE(byte_index) = '1' ) then
                slv_reg9(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
              end if;
            end loop;
          when "0000000000100000" =>
            for byte_index in 0 to (C_SLV_DWIDTH/8)-1 loop
              if ( Bus2IP_BE(byte_index) = '1' ) then
                slv_reg10(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
              end if;
            end loop;
          when "0000000000010000" =>
            for byte_index in 0 to (C_SLV_DWIDTH/8)-1 loop
              if ( Bus2IP_BE(byte_index) = '1' ) then
                slv_reg11(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
              end if;
            end loop;
          when "0000000000001000" =>
            for byte_index in 0 to (C_SLV_DWIDTH/8)-1 loop
              if ( Bus2IP_BE(byte_index) = '1' ) then
                slv_reg12(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
              end if;
            end loop;
          when "0000000000000100" =>
            for byte_index in 0 to (C_SLV_DWIDTH/8)-1 loop
              if ( Bus2IP_BE(byte_index) = '1' ) then
                slv_reg13(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
              end if;
            end loop;
          when "0000000000000010" =>
            for byte_index in 0 to (C_SLV_DWIDTH/8)-1 loop
              if ( Bus2IP_BE(byte_index) = '1' ) then
                slv_reg14(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
              end if;
            end loop;
          when "0000000000000001" =>
            for byte_index in 0 to (C_SLV_DWIDTH/8)-1 loop
              if ( Bus2IP_BE(byte_index) = '1' ) then
                slv_reg15(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
              end if;
            end loop;
          when others => -- write the registers when the Done signal is asserted
            if DONE = '1' then
               if (SLOT_VALID (4) = '1' AND SLOT_VALID (3) = '1') then
                  ADDR (C_SLV_DWIDTH - 15 to C_SLV_DWIDTH - 9)  <= ADDR_OUT;
                  ADDR (0 to C_SLV_DWIDTH - 16)  <= (others => '0');
               end if;
               
               TAG_AND_ID (C_SLV_DWIDTH - 13 to C_SLV_DWIDTH - 9) <= SLOT_VALID;
               TAG_AND_ID (0 to C_SLV_DWIDTH - 14) <= (others => '0');
               
               -- Set bit 31 of STATUS to '1'
               STATUS (C_SLV_DWIDTH - 2) <= '1';
               STATUS (C_SLV_DWIDTH - 4 to C_SLV_DWIDTH - 3) <= SLOT_REQ;
               STATUS (0 to C_SLV_DWIDTH - 5) <= (others => '0');
               
               if (SLOT_VALID (4) = '1' AND SLOT_VALID (2) = '1') then
                  CONFIG_DATA (0 to C_SLV_DWIDTH - 17)  <= DATA_OUT;
               --CONFIG_DATA (0 to C_SLV_DWIDTH - 17)  <= ( others => '0');
               end if;
               
               --if (SLOT_VALID (4) = '1' AND SLOT_VALID (1) = '1') then
                  PCM_IN_L (C_SLV_DWIDTH - 20 to C_SLV_DWIDTH - 1)    <= PCM_IN_L_REC;
                  PCM_IN_L (0 to C_SLV_DWIDTH - 21)    <= (others => '0');
               --end if;
               
               --if (SLOT_VALID (4) = '1' AND SLOT_VALID (0) = '1') then
                  PCM_IN_R (C_SLV_DWIDTH - 20 to C_SLV_DWIDTH - 1)    <= PCM_IN_R_REC;
                  PCM_IN_R (0 to C_SLV_DWIDTH - 21)    <= (others => '0');
               --end if;
      
               -- self-set the RNW signal
               --CONTROL (C_SLV_DWIDTH - 1) <= '0';
               CONTROL (C_SLV_DWIDTH - 2) <= '1';
               --CONTROL (0 to C_SLV_DWIDTH - 3)  <= (others => '0');
             end if;
           if STC_LOAD = Clear_Load_Status then
               STATUS (C_SLV_DWIDTH - 1) <= '0';
               CONTROL (C_SLV_DWIDTH - 1) <= '0';
           elsif CONTROL (C_SLV_DWIDTH - 1) = '1' then
               STATUS (C_SLV_DWIDTH - 1) <= '1';               
           end if;
        end case;
      end if;
    end if;

  end process SLAVE_REG_WRITE_PROC;

  -- implement slave model software accessible register(s) read mux
  SLAVE_REG_READ_PROC : process( slv_reg_read_sel, ADDR, CONFIG_DATA, TAG_AND_ID, PCM_OUT_L, PCM_OUT_R, PCM_IN_L, PCM_IN_R, CONTROL, STATUS, slv_reg9, slv_reg10, slv_reg11, slv_reg12, slv_reg13, slv_reg14, slv_reg15 ) is
  begin

    case slv_reg_read_sel is
      when "1000000000000000" => slv_ip2bus_data <= ADDR;
      when "0100000000000000" => slv_ip2bus_data <= CONFIG_DATA;
      when "0010000000000000" => slv_ip2bus_data <= TAG_AND_ID;
      when "0001000000000000" => slv_ip2bus_data <= PCM_OUT_L;
      when "0000100000000000" => slv_ip2bus_data <= PCM_OUT_R;
      when "0000010000000000" => slv_ip2bus_data <= PCM_IN_L;
      when "0000001000000000" => slv_ip2bus_data <= PCM_IN_R;
      when "0000000100000000" => slv_ip2bus_data <= CONTROL;
      when "0000000010000000" => slv_ip2bus_data <= STATUS;
      when "0000000001000000" => slv_ip2bus_data <= slv_reg9;
      when "0000000000100000" => slv_ip2bus_data <= slv_reg10;
      when "0000000000010000" => slv_ip2bus_data <= slv_reg11;
      when "0000000000001000" => slv_ip2bus_data <= slv_reg12;
      when "0000000000000100" => slv_ip2bus_data <= slv_reg13;
      when "0000000000000010" => slv_ip2bus_data <= slv_reg14;
      when "0000000000000001" => slv_ip2bus_data <= slv_reg15;
      when others => slv_ip2bus_data <= (others => '0');
    end case;

  end process SLAVE_REG_READ_PROC;

  ------------------------------------------
  -- Example code to drive IP to Bus signals
  ------------------------------------------
  IP2Bus_Data  <= slv_ip2bus_data when slv_read_ack = '1' else
                  (others => '0');

  IP2Bus_WrAck <= slv_write_ack;
  IP2Bus_RdAck <= slv_read_ack;
  IP2Bus_Error <= '0';

end IMP;
